@@ -366,6 +366,8 @@ class Attention(nn.Module):
 
         self.norm = LayerNorm(dim)
 
+        self.null_attn_bias = nn.Parameter(torch.randn(heads))
+
         self.null_kv = nn.Parameter(torch.randn(2, dim_head))
         self.to_q = nn.Linear(dim, inner_dim, bias = False)
         self.to_kv = nn.Linear(dim, dim_head * 2, bias = False)
@@ -412,7 +414,8 @@ class Attention(nn.Module):
         # relative positional encoding (T5 style)
 
         if exists(attn_bias):
-            attn_bias = F.pad(attn_bias, (1, 0), value = 0.)
+            null_attn_bias = repeat(self.null_attn_bias, 'h -> h n 1', n = n)
+            attn_bias = torch.cat((null_attn_bias, attn_bias), dim = -1)
             sim = sim + attn_bias
 
         # masking
@@ -458,6 +461,15 @@ def Conv2d(dim_in, dim_out, kernel, stride = 1, padding = 0, **kwargs):
 
     return nn.Conv3d(dim_in, dim_out, kernel, stride = stride, padding = padding, **kwargs)
 
+class Pad(nn.Module):
+    def __init__(self, padding, value = 0.):
+        super().__init__()
+        self.padding = padding
+        self.value = value
+
+    def forward(self, x):
+        return F.pad(x, self.padding, value = self.value)
+
 # decoder
 
 def Upsample(dim, dim_out = None):
@@ -1192,7 +1204,9 @@ class Unet3D(nn.Module):
 
         # temporal attention - attention across video frames
 
-        temporal_peg = lambda dim: Residual(nn.Conv3d(dim, dim, (3, 1, 1), padding = (1, 0, 0), groups = dim))
+        temporal_peg_padding = (0, 0, 0, 0, 2, 0) if time_causal_attn else (0, 0, 0, 0, 1, 1)
+        temporal_peg = lambda dim: Residual(nn.Sequential(Pad(temporal_peg_padding), nn.Conv3d(dim, dim, (3, 1, 1), groups = dim)))
+
         temporal_attn = lambda dim: EinopsToAndFrom('b c f h w', '(b h w) f c', Residual(Attention(dim, **{**attn_kwargs, 'causal': time_causal_attn})))
 
         # temporal attention relative positional encoding
